import NodeType from './NodeType';
import Document from '../document/Document';
import EventTarget from '../../../event/EventTarget';
import MutationObserverListener from '../../../mutation-observer/MutationListener';
import Event from '../../../event/Event';
/**
 * Node
 */
export default class Node extends EventTarget {
    static ownerDocument: Document;
    ownerDocument: Document;
    parentNode: Node;
    readonly nodeType: NodeType;
    readonly childNodes: Node[];
    protected _isConnected: boolean;
    protected _observers: MutationObserverListener[];
    /**
     * Constructor.
     */
    constructor();
    /**
     * "true" if connected to DOM.
     *
     * @return {boolean} "true" if connected.
     */
    get isConnected(): boolean;
    /**
     * Sets the connected state.
     *
     * @param {boolean} isConnected "true" if connected.
     */
    set isConnected(isConnected: boolean);
    /**
     * Node value.
     *
     * @return {string} Node value.
     */
    get nodeValue(): string;
    /**
     * Node name.
     *
     * @return {string} Node name.
     */
    get nodeName(): string;
    /**
     * Previous sibling.
     *
     * @return {Node} Node.
     */
    get previousSibling(): Node;
    /**
     * Next sibling.
     *
     * @return {Node} Node.
     */
    get nextSibling(): Node;
    /**
     * First child.
     *
     * @return {Node} Node.
     */
    get firstChild(): Node;
    /**
     * Last child.
     *
     * @return {Node} Node.
     */
    get lastChild(): Node;
    /**
     * Connected callback.
     */
    connectedCallback?(): void;
    /**
     * Disconnected callback.
     */
    disconnectedCallback?(): void;
    /**
     * Returns "true" if the node has attributes.
     *
     * @return {boolean} "true" if the node has attributes.
     */
    hasAttributes(): boolean;
    /**
     * Clones a node.
     *
     * @param {boolean} [deep=true] "false" to not clone deep.
     * @return {Node} Cloned node.
     */
    cloneNode(deep?: boolean): Node;
    /**
     * Append a child node to childNodes.
     *
     * @param  {Node} node Node to append.
     * @return {Node} Appended node.
     */
    appendChild(node: Node): Node;
    /**
     * Remove Child element from childNodes array.
     *
     * @param {Node} node Node to remove
     */
    removeChild(node: Node): void;
    /**
     * Inserts a node before another.
     *
     * @param {Node} newNode Node to insert.
     * @param {Node} referenceNode Node to insert before.
     * @return {Node} Inserted node.
     */
    insertBefore(newNode: Node, referenceNode: Node): Node;
    /**
     * Replaces a node with another.
     *
     * @param {Node} newChild New child.
     * @param {Node} oldChild Old child.
     * @return {Node} Replaced node.
     */
    replaceChild(newChild: Node, oldChild: Node): Node;
    /**
     * Dispatches an event.
     *
     * @override
     * @param {Event} event Event.
     * @return {boolean} The return value is false if event is cancelable and at least one of the event handlers which handled this event called Event.preventDefault()
     */
    dispatchEvent(event: Event): boolean;
    /**
     * Observeres the node.
     * Used by MutationObserver, but it is not part of the HTML standard.
     *
     * @param {MutationObserverListener} listener Listener.
     */
    _observe(listener: MutationObserverListener): void;
    /**
     * Stops observing the node.
     * Used by MutationObserver, but it is not part of the HTML standard.
     *
     * @param {MutationObserverListener} listener Listener.
     */
    _unobserve(listener: MutationObserverListener): void;
}
